#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>

int main(void)
{
  pid_t pid;
  int status;
  char initpath[256];

  pid = vfork ();
  if (pid == 0) {
    execl ("/sbin/ldconfig", "/sbin/ldconfig", NULL);
    exit (1);
  } else if (pid < 0) {
    perror ("fork failed\n");
    exit (1);
  }
  if (waitpid (0, &status, 0) != pid || !WIFEXITED (status)) {
    fprintf(stderr, "ldconfig died\n");
    exit (1);
  }
  if (WEXITSTATUS (status))
    exit (WEXITSTATUS (status));

  /* Check if telinit is available and the init fifo as well.  */
  if (access ("/sbin/telinit", X_OK) || access ("/dev/initctl", F_OK))
    exit (0);
  /* Check if we are not inside of some chroot, because we'd just
     timeout and leave /etc/initrunlvl.  */
  if (readlink ("/proc/1/exe", initpath, 256) <= 0 ||
      readlink ("/proc/1/root", initpath, 256) <= 0)
    exit (0);
  pid = vfork ();
  if (pid == 0) {
    execl ("/sbin/telinit", "/sbin/telinit", "u", NULL);
    exit (1);
  } else if (pid < 0) {
    perror ("fork failed\n");
    exit (1);
  }
  if (waitpid (0, &status, 0) != pid || !WIFEXITED (status)) {
    fprintf(stderr, "telinit died\n");
    exit (1);
  }
  exit(0);
}
